package main

import (
	"flag"
	"fmt"
	"log"
	"net/http"
	"os"
	"path"
	"path/filepath"
	"text/template"

	"srs.epita.fr/fic-server/admin/api"
	"srs.epita.fr/fic-server/libfic"
)

var PKIDir string
var SubmissionDir string
var StaticDir string

func main() {
	var bind = flag.String("bind", "127.0.0.1:8081", "Bind port/socket")
	var dsn = flag.String("dsn", "fic:fic@/fic", "DSN to connect to the MySQL server")
	var baseURL = flag.String("baseurl", "/", "URL prepended to each URL")
	flag.StringVar(&SubmissionDir, "submission", "./submissions/", "Base directory where save submissions")
	flag.StringVar(&PKIDir, "pki", "./pki/", "Base directory where found PKI scripts")
	flag.StringVar(&StaticDir, "static", "./htdocs-admin/", "Directory containing static files")
	flag.StringVar(&api.TeamsDir, "teams", "./TEAMS", "Base directory where save teams JSON files")
	flag.StringVar(&fic.FilesDir, "files", "./FILES/", "Base directory where found challenges files, local part")
	flag.StringVar(&api.CloudDAVBase, "clouddav", "https://srs.epita.fr/owncloud/remote.php/webdav/FIC 2016",
		"Base directory where found challenges files, cloud part")
	flag.StringVar(&api.CloudUsername, "clouduser", "fic", "Username used to sync")
	flag.StringVar(&api.CloudPassword, "cloudpass", "", "Password used to sync")
	flag.BoolVar(&api.RapidImport, "rapidimport", false, "Don't try to reimport an existing file")
	flag.BoolVar(&fic.OptionalDigest, "optionaldigest", false, "Is the digest required when importing files?")
	flag.Parse()

	log.SetPrefix("[admin] ")

	var err error
	log.Println("Checking paths...")
	if StaticDir, err = filepath.Abs(StaticDir); err != nil {
		log.Fatal(err)
	}
	if fic.FilesDir, err = filepath.Abs(fic.FilesDir); err != nil {
		log.Fatal(err)
	}
	if PKIDir, err = filepath.Abs(PKIDir); err != nil {
		log.Fatal(err)
	}
	if SubmissionDir, err = filepath.Abs(SubmissionDir); err != nil {
		log.Fatal(err)
	}
	if api.TeamsDir, err = filepath.Abs(api.TeamsDir); err != nil {
		log.Fatal(err)
	}
	if fic.FilesDir, err = filepath.Abs(fic.FilesDir); err != nil {
		log.Fatal(err)
	}
	if *baseURL != "/" {
		tmp := path.Clean(*baseURL)
		baseURL = &tmp
	} else {
		tmp := ""
		baseURL = &tmp
	}

	if err := os.Chdir(PKIDir); err != nil {
		log.Fatal("Unable to enter PKI directory at: ", err)
	}

	log.Println("Opening database...")
	if err := fic.DBInit(fmt.Sprintf("%s?parseTime=true", *dsn)); err != nil {
		log.Fatal("Cannot open the database: ", err)
	}
	defer fic.DBClose()

	log.Println("Creating database...")
	if err := fic.DBCreate(); err != nil {
		log.Fatal("Cannot create database: ", err)
	}

	log.Println("Changing base URL to", *baseURL,"...")
	if file, err := os.OpenFile(path.Join(StaticDir, "index.html"), os.O_CREATE|os.O_WRONLY|os.O_TRUNC, os.FileMode(0644)); err != nil {
		log.Println("Unable to open index.html: ", err)
	} else if indexTmpl, err := template.New("index").Parse(indextpl); err != nil {
		log.Println("Cannot create template: ", err)
	} else if err := indexTmpl.Execute(file, map[string]string{"urlbase": path.Clean(path.Join(*baseURL + "/", "nuke"))[:len(path.Clean(path.Join(*baseURL + "/", "nuke"))) - 4]}); err != nil {
		log.Println("An error occurs during template execution: ", err)
	}


	log.Println(fmt.Sprintf("Ready, listening on %s", *bind))
	if err := http.ListenAndServe(*bind, http.StripPrefix(*baseURL, api.Router())); err != nil {
		log.Fatal("Unable to listen and serve: ", err)
	}
}
